%--------------------------------------------------------------------------
function Main()
    % Each run is 426 seconds long, 213 TRs (7.1 minutes)

	% Setup for run
	addpath('Tools');
	prefs = Preferences();
	prefs.studyName = 'alphabetDigits';
	prefs.runNum    = input('Run number: ');
	[win,fid1,fid2] = Initialize(prefs); 
	
	% Get information about blocks and events in those blocks
	b = BlockInfo();
	b = EventInfo(b, prefs.studyName, prefs.runNum); 
	
	% Wait for trigger
	triggerTime = WaitForTrigger(win, prefs, 'Press button for jiggling character');
    Screen('TextSize', win, 90);

	for i=1:length(b.conds)		
		start = GetSecs;
		
		% Normalize the onsets within this block to the trigger time
		b.events(i).onsets = b.events(i).onsets + b.onsets(i) + triggerTime;
		
		% Present block
		PresentBlock(b.events(i), win, prefs, fid2, triggerTime);
		
		% Output information about this block to screen and file
		s = sprintf('%.2f\t%d\t%.2f\t%.2f\t%s\n', start - triggerTime, ...
			b.conds(i), b.durs(i), 1, b.condNames{b.conds(i)+1});
		fprintf(fid1, s); fprintf('B: %s', s);
	end
	fclose(fid1); fclose(fid2);
    sca;
			
%--------------------------------------------------------------------------	
function PresentBlock(events, win, prefs, fid, trigTime)

	% Initialize variables
	startBlock = GetSecs;
	rct = Screen('Rect', win);
        
	for j=1:length(events.conds)
		% Initialize for event
		start = GetSecs;
        rt1 = 0; key1 = 0; key2 = 0; rt2 = 0;
		
		if events.conds(j) ~= 0
            % Condition 1 (no jiggle), offset is always 0
            offsets = [0 0 0 0 0 0];
            % Condition 2, setup jiggles
            if events.conds(j) == 2
                offsets = [4 0 -4 0 4 0] .* sign(randn);
            end
            
            for o = offsets
                % Draw letter/digit
                DrawCenteredText(win, events.letters(j), [], o, -60);
                Screen('FillOval', win, prefs.foreColor, CenterRect([0 0 8 8], rct));            
                Screen('Flip',win);
            end
            
            % Look for keys until 200ms before the end of this event
            [key1, rt1] = WaitLookingForKeys(prefs.responseKeys, ...
				events.onsets(j+1) - 0.200, start);
        end
            
        % Blank screen and wait for keys until the end of this event
        Screen('FillRect', win, prefs.backColor);
		Screen('FillOval', win, prefs.foreColor, CenterRect([0 0 8 8], rct));            
        Screen('Flip',win);        
        [key2, rt2] = WaitLookingForKeys(prefs.responseKeys, ...
				events.onsets(j+1), start);
            
		% Output information about this event to screen and file
		s = sprintf('%.2f\t%d\t%.2f\t%.2f\t%s\t%d\t%f\t%d\t%0.3f\t%s\n', ...
		  start - trigTime, events.conds(j), events.durs(j), 1, ...
		  events.condNames{events.conds(j)+1}, key1, rt1, key2, rt2, events.letters(j));	
		fprintf(fid, s); fprintf('  E: %s', s);
	end
	
%--------------------------------------------------------------------------	
function b = EventInfo(b, studyName, runNum)

	% Fill in the events for each block
	for i=1:length(b.conds)
		b.events(i).condNames{0 + 1} = 'NULL';
	
		% For fixation blocks, make a single event with a blank character
		% for the entire duration of the block
		if b.conds(i) == 0
			b.events(i).conds   = 0;
			b.events(i).durs    = b.durs(i);
			b.events(i).onsets  = [0 b.durs(i)];
            b.events(i).letters = ' ';
			continue;
		end
		
		% Names of event types in this block (conditions 1 and 2)
		b.events(i).condNames{1 + 1} = 'Character';
		b.events(i).condNames{2 + 1} = 'JiggledCharacter';
        
        % Setup stimuli to use
        letterOrder = Shuffle('ABCDEFGHIJKLMNOPQRSTUVWXYZ');
        if b.conds(i) == 2
            letterOrder = Shuffle('01234567890123456789');
        end
        
		% Set what letters to show for each event, how long the event
		% should be, and when it should start
        b.events(i).letters = letterOrder(1:20); 
		b.events(i).conds   = repmat(1, [1 20]);
        b.events(i).durs    = repmat(0.800, [1 20]);
        b.events(i).onsets  = [0 cumsum(b.events(i).durs)];
		
		% Make characters jiggle twice per block by setting them to
		% condition 2, the jiggled character condition
		b.events(i).conds(Randi(20))     = 2;
        b.events(i).conds(Randi(20))     = 2;   
	end
	
	
%--------------------------------------------------------------------------	
function b = BlockInfo()	
	% How long is each type of block
	lengthFixation   = 10;
	lengthMainBlocks = 16;
	
	% How many main blocks of each condition (8 letter blocks and 8 digit
	% blocks, each 16 seconds long per the line above)
	condition(1) = 8;
	condition(2) = 8;
	
	% What should we call these conditions in the output (for conditions 0,
	% 1 and 2)
	b.condNames{0 + 1}  = 'NULL'; 
	b.condNames{1 + 1}  = 'Letters'; 
	b.condNames{2 + 1}  = 'Digits';

	% Create block order (fixations sandwiching each block)
	numBlocksTotal = sum(condition).*2+1;
	b.conds = zeros(1,numBlocksTotal);
	blocksToInsert = [];
	for i=1:length(condition)
	  blocksToInsert(end+1:end+condition(i)) = repmat(i,1,condition(i));
	end
	b.conds(2:2:end) = Shuffle(blocksToInsert);
	
	% Creation durations and onsets of these blocks
	b.durs = zeros(1,numBlocksTotal) + lengthFixation;
	b.durs(2:2:end) = lengthMainBlocks;
	b.onsets = [0 cumsum(b.durs)];

